@@ -317,21 +317,22 @@ test_that("delta models work with different main effects", {
   )
 
   # diff random intercepts throw an error for now
-  expect_error(
-    {
-      pcod_2011$fyear <- as.factor(pcod_2011$year)
-      fit <- sdmTMB(
-        formula = list(density ~ 1 + (1 | fyear), density ~ 1),
-        data = pcod_2011,
-        mesh = mesh,
-        spatial = "off",
-        family = delta_gamma()
-      )
-    },
-    regexp = "random intercepts"
-  )
+  # expect_error(
+  #   {
+  #     pcod_2011$fyear <- as.factor(pcod_2011$year)
+  #     fit <- sdmTMB(
+  #       formula = list(density ~ 1 + (1 | fyear), density ~ 1),
+  #       data = pcod_2011,
+  #       mesh = mesh,
+  #       spatial = "off",
+  #       family = delta_gamma()
+  #     )
+  #   },
+  #   regexp = "random intercepts"
+  # )
 
   # OK:
+  pcod_2011$fyear <- as.factor(pcod_2011$year)
   fit <- sdmTMB(
     formula = list(density ~ 1 + (1 | fyear), density ~ 1 + (1 | fyear)),
     data = pcod_2011,
